package com.cicdi.notify;

import com.cicdi.notify.template.Template;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 通知器管理器抽象实现
 * <p>实现了全部的接口，抽象的目的是为了方便实现Spring的自动装配回接口，实现服务商的自动注册</p>
 *
 * @author xueye
 */
public abstract class AbstractNotifierManager implements NotifierManager {

    protected final Map<String, Map<String, NotifierProvider>> providers = new ConcurrentHashMap<>();

    protected final Map<String, Notifier> notifiers = new ConcurrentHashMap<>();

    protected final NotifyConfigManager configManager;

//    private EventBus eventBus;

    public AbstractNotifierManager(NotifyConfigManager manager/*, EventBus eventBus*/) {
        this.configManager = manager;
//        this.eventBus = eventBus;
    }

    @Override
    public <T extends Template> Notifier<T> getNotifier(NotifyType type, String id) {
        Notifier<T> notifier;
        if (null == notifiers.get(id)) {
            notifier = createNotifier(getProperties(type, id));
        } else {
            notifier = notifiers.get(id);
        }
        return notifier;
    }

    @Override
    public <T extends Template> Notifier<T> createNotifier(NotifierProperties properties) {
        Map<String, NotifierProvider> providerMap = providers.get(properties.getType());
        if (null == providerMap) {
            throw new UnsupportedOperationException("不支持的通知类型: " + properties.getType());
        }
        NotifierProvider provider = providerMap.get(properties.getProvider());
        if (null == provider) {
            throw new UnsupportedOperationException("不支持的服务商: " + properties.getProvider());
        }
        Notifier notifierNew = provider.createNotifier(properties);
        Notifier notifierOld = notifiers.put(properties.getId(), notifierNew);
        if (null != notifierOld) {
            notifierOld.close();
        }
        return notifierNew;
    }

    @Override
    public void register(NotifierProvider provider) {
        providers.computeIfAbsent(provider.getType().getId(), ignore -> new ConcurrentHashMap<>())
                .put(provider.getProvider().getId(), provider);
    }

    @Override
    public void reload(String id) {
        Notifier notifier = notifiers.remove(id);
        if (null != notifier) {
            notifier.close();
        }
    }

    /**
     * 根据通知配置属性ID获取通知配置属性
     *
     * @param notifyType 通知类型
     * @param id         通知配置属性ID
     * @return 通知配置属性
     */
    protected NotifierProperties getProperties(NotifyType notifyType, String id) {
        return configManager.getNotifyConfig(notifyType, id);
    }

}
